@@ -31,11 +31,13 @@ class MatMul(Function, Module):
             else:
                 return xp.swapaxes(arr, -2, -1)  # Swap last two axes for higher dims
         if A.requires_grad:
-            # grad_A = grad_output @ B.T
-            grad_A = xp.matmul(grad_output, _transpose(B.data))
+            # Match grad dtype to operand to avoid upcasting large tensors
+            go = grad_output.astype(B.data.dtype, copy=False)
+            grad_A = xp.matmul(go, _transpose(B.data))
         if B.requires_grad:
-            # grad_B = A.T @ grad_output  
-            grad_B = xp.matmul(_transpose(A.data), grad_output)
+            # Match grad dtype to operand to avoid upcasting large tensors
+            go = grad_output.astype(A.data.dtype, copy=False)
+            grad_B = xp.matmul(_transpose(A.data), go)
         return grad_A, grad_B
 
 
@@ -79,7 +81,9 @@ class TensorDot(Function, Module):
         grad_A = grad_B = None
     
         if A.requires_grad:
-            grad_A = xp.tensordot(grad_output, B.data, 
+            # Cast incoming grad to B's dtype to prevent upcasting B (often large/strided)
+            go = grad_output.astype(B.data.dtype, copy=False)
+            grad_A = xp.tensordot(go, B.data,
                                 axes=[list(range(len(A_free), len(A_free) + len(B_free))), B_free])
             perm = [0] * A.ndim
             for i, ax in enumerate(A_free):
@@ -89,7 +93,9 @@ class TensorDot(Function, Module):
             grad_A = xp.transpose(grad_A, perm)
         
         if B.requires_grad:
-            grad_B = xp.tensordot(A.data, grad_output,
+            # Cast incoming grad to A's dtype to prevent upcasting A
+            go = grad_output.astype(A.data.dtype, copy=False)
+            grad_B = xp.tensordot(A.data, go,
                                 axes=[A_free, list(range(len(A_free)))])
             perm = [0] * B.ndim
             for i, ax in enumerate(B_axes):
